@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
+# Copyright (c) 2001, 2002, 2009 Zope Corporation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -13,7 +13,7 @@
 ##############################################################################
 """Component Architecture Tests
 
-$Id: tests.py 72389 2007-02-06 05:14:32Z gary $
+$Id: tests.py 97939 2009-03-12 00:01:36Z nadako $
 """
 import re
 import unittest
@@ -36,12 +36,6 @@ import zope.component.globalregistry
 # we should obviously make this more explicit
 import zope.component.event
 
-import zope.deferredimport
-zope.deferredimport.deprecated(
-    "Use zope.component.testing instead",
-    placelesssetup = "zope.component.testing",
-    )
-
 class I1(interface.Interface):
     pass
 class I2(interface.Interface):
@@ -951,6 +945,98 @@ def test_multi_handler_unregistration():
     | Factory 2 is here
     """
 
+def test_next_utilities():
+    """
+    It is common for a utility to delegate its answer to a utility
+    providing the same interface in one of the component registry's
+    bases. Let's first create a global utility::
+    
+      >>> import zope.interface
+      >>> class IMyUtility(zope.interface.Interface):
+      ...     pass
+    
+      >>> class MyUtility(ConformsToIComponentLookup):
+      ...     zope.interface.implements(IMyUtility)
+      ...     def __init__(self, id, sm):
+      ...         self.id = id
+      ...         self.sitemanager = sm
+      ...     def __repr__(self):
+      ...         return "%s('%s')" % (self.__class__.__name__, self.id)
+
+      >>> from zope.component import getGlobalSiteManager
+      >>> gsm = getGlobalSiteManager()
+    
+      >>> gutil = MyUtility('global', gsm)
+      >>> gsm.registerUtility(gutil, IMyUtility, 'myutil')
+    
+    Now, let's create two registries and set up the bases hierarchy::
+    
+      >>> from zope.component.registry import Components
+      >>> sm1 = Components('sm1', bases=(gsm, ))
+      >>> sm1_1 = Components('sm1_1', bases=(sm1, ))
+    
+    Now we create two utilities and insert them in our folder hierarchy:
+    
+      >>> util1 = MyUtility('one', sm1)
+      >>> sm1.registerUtility(util1, IMyUtility, 'myutil')
+      >>> IComponentLookup(util1) is sm1
+      True
+
+      >>> util1_1 = MyUtility('one-one', sm1_1)
+      >>> sm1_1.registerUtility(util1_1, IMyUtility, 'myutil')
+      >>> IComponentLookup(util1_1) is sm1_1
+      True
+    
+    Now, if we ask `util1_1` for its next available utility we get the
+    ``one`` utility::
+    
+      >>> from zope.component import getNextUtility
+      >>> getNextUtility(util1_1, IMyUtility, 'myutil')
+      MyUtility('one')
+    
+    Next we ask `util1` for its next utility and we should get the global version:
+    
+      >>> getNextUtility(util1, IMyUtility, 'myutil')
+      MyUtility('global')
+    
+    However, if we ask the global utility for the next one, an error is raised
+    
+      >>> getNextUtility(gutil, IMyUtility,
+      ...                     'myutil') #doctest: +NORMALIZE_WHITESPACE
+      Traceback (most recent call last):
+      ...
+      ComponentLookupError:
+      No more utilities for <InterfaceClass zope.component.tests.IMyUtility>,
+      'myutil' have been found.
+    
+    You can also use `queryNextUtility` and specify a default:
+    
+      >>> from zope.component import queryNextUtility
+      >>> queryNextUtility(gutil, IMyUtility, 'myutil', 'default')
+      'default'
+    
+    Let's now ensure that the function also works with multiple registries. First
+    we create another base registry:
+    
+      >>> myregistry = Components()
+    
+    We now set up another utility into that registry:
+    
+      >>> custom_util = MyUtility('my_custom_util', myregistry)
+      >>> myregistry.registerUtility(custom_util, IMyUtility, 'my_custom_util')
+    
+    We add it as a base to the local site manager:
+    
+      >>> sm1.__bases__ = (myregistry,) + sm1.__bases__
+    
+    Both the ``myregistry`` and global utilities should be available:
+    
+      >>> queryNextUtility(sm1, IMyUtility, 'my_custom_util')
+      MyUtility('my_custom_util')
+      >>> queryNextUtility(sm1, IMyUtility, 'myutil')
+      MyUtility('global')
+    """
+
 class StandaloneTests(unittest.TestCase):
     def testStandalone(self):
         import subprocess
@@ -966,7 +1052,6 @@ class StandaloneTests(unittest.TestCase):
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.STDOUT,
                                    stdin=subprocess.PIPE)
-
         pickle.dump(sys.path, process.stdin)
         process.stdin.close()
 
@@ -977,13 +1062,132 @@ class StandaloneTests(unittest.TestCase):
                 raise # TODO verify sanity of a pass on EINTR :-/
         lines = process.stdout.readlines()
         process.stdout.close()
+        success = True
+        # Interpret the result: We scan the output from the end backwards
+        # until we find either a line that says 'OK' (which means the tests
+        # ran successfully) or a line that starts with quite a few dashes
+        # (which means we didn't find a line that says 'OK' within the summary
+        # of the test runner and the tests did not run successfully.)
         for l in reversed(lines):
             l = l.strip()
-            if l:
-                if not l.endswith('OK'):
-                    self.fail(''.join(lines))
-                else:
-                    break
+            if not l:
+                continue
+            if l.startswith('-----'):
+                break
+            if l.endswith('OK'):
+                sucess = True
+        if not success:
+            self.fail(''.join(lines))
+
+class HookableTests(unittest.TestCase):
+
+    def test_ctor_no_func(self):
+        from zope.component.hookable import hookable
+        self.assertRaises(TypeError, hookable)
+
+    def test_ctor_simple(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        hooked = hookable(foo)
+        self.failUnless(hooked.original is foo)
+        self.failUnless(hooked.implementation is foo)
+
+    def test_ctor_extra_arg(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        self.assertRaises(TypeError, hookable, foo, foo)
+
+    def test_ctor_extra_arg(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        self.assertRaises(TypeError, hookable, foo, nonesuch=foo)
+
+    def test_sethook(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        def bar():
+            pass
+        hooked = hookable(foo)
+        hooked.sethook(bar)
+        self.failUnless(hooked.original is foo)
+        self.failUnless(hooked.implementation is bar)
+
+    def test_reset(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        def bar():
+            pass
+        hooked = hookable(foo)
+        hooked.sethook(bar)
+        hooked.reset()
+        self.failUnless(hooked.original is foo)
+        self.failUnless(hooked.implementation is foo)
+
+    def test_cant_assign_original(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        def bar():
+            pass
+        hooked = hookable(foo)
+        try:
+            hooked.original = bar
+        except TypeError:
+            pass
+        except AttributeError:
+            pass
+        else:
+            self.fail('Assigned original')
+
+    def test_cant_delete_original(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        hooked = hookable(foo)
+        try:
+            del hooked.original
+        except TypeError:
+            pass
+        except AttributeError:
+            pass
+        else:
+            self.fail('Deleted original')
+
+    def test_cant_assign_original(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        def bar():
+            pass
+        hooked = hookable(foo)
+        try:
+            hooked.implementation = bar
+        except TypeError:
+            pass
+        except AttributeError:
+            pass
+        else:
+            self.fail('Assigned implementation')
+
+    def test_readonly_original(self):
+        from zope.component.hookable import hookable
+        def foo():
+            pass
+        hooked = hookable(foo)
+        try:
+            del hooked.implementation
+        except TypeError:
+            pass
+        except AttributeError:
+            pass
+        else:
+            self.fail('Deleted implementation')
+
 
 def setUpRegistryTests(tests):
     setUp()
@@ -1003,12 +1207,15 @@ def clearZCML(test=None):
 def test_suite():
     checker = renormalizing.RENormalizing([
         (re.compile('at 0x[0-9a-fA-F]+'), 'at <SOME ADDRESS>'),
+        (re.compile(r"<type 'exceptions.(\w+)Error'>:"),
+                    r'exceptions.\1Error:'),
         ])
 
     return unittest.TestSuite((
         doctest.DocTestSuite(setUp=setUp, tearDown=tearDown),
         doctest.DocTestSuite('zope.component.interface',
                              setUp=setUp, tearDown=tearDown),
+        doctest.DocTestSuite('zope.component.nexttesting'),
         doctest.DocFileSuite('README.txt',
                              setUp=setUp, tearDown=tearDown),
         doctest.DocFileSuite('socketexample.txt',
@@ -1020,7 +1227,7 @@ def test_suite():
                              tearDown=tearDownRegistryTests),
         doctest.DocFileSuite('event.txt',
                              setUp=setUp, tearDown=tearDown),
-        doctest.DocFileSuite('zcml.txt',
+        doctest.DocFileSuite('zcml.txt',checker=checker,
                              setUp=setUp, tearDown=tearDown),
         unittest.makeSuite(StandaloneTests),
         ))
